;*************************************************************************	
;* DEVICE DEFINITION FILE - serverBOT
;* DEVICE: ATMEL AT90S8535
;*************************************************************************	
.device AT90S8535

;-= I/O Register Definitions =-
.equ	SREG	=$3F	; Status Register
.equ	SPH	=$3E	; Stack Pointer High
.equ	SPL	=$3D	; Stack Pointer Low
.equ	GIMSK	=$3B	; General Interrupt Mask Register
.equ	GIFR	=$3A	; General Interrupt Flag Register
.equ	TIMSK	=$39	; Timer/Counter Interrupt Mask Register
.equ	TIFR	=$38	; Timer/Counter Interrupt Flag Register
.equ	MCUCR	=$35	; MCU Control Register
;* TIMER0 I/O REGISTERS
.equ	TCCR0	=$33	; Timer/Counter 0 Control Register
.equ	TCNT0	=$32	; Timer Counter 0
;* TIMER1 I/0 REGISTERS
.equ	TCCR1A	=$2F	; Timer/Counter 1 Control Register A
.equ	TCCR1B	=$2E	; Timer/Counter 1 Control Register B
.equ	TCNT1H	=$2D	; Timer/Counter 1 High Byte
.equ	TCNT1L	=$2C	; Timer/Counter 1 Low Byte
.equ	OCR1AH	=$2B	; Timer Output Compare 1A High Byte
.equ	OCR1AL	=$2A	; Timer Output Compare 1A Low Byte
.equ	OCR1BH	=$29	; Timer Output Compare 1B High Byte
.equ	OCR1BL	=$28	; Timer Output Compare 1B Low Byte
;* ITIMER2 I/O REGISTERS
.equ	TCCR2	=$25	; Timer/Counter 2 Control Register
.equ	TCNT2	=$24	; Timer/Counter 2
.equ	OCR2	=$23	; Timer Output Compare 2
.equ	ASSR	=$22	; Asynchronous Status Register 
;* EEPROM
.equ	EEARH	=$1F	; EEPROM Address Register High
.equ	EEARL	=$1E	; EEPROM Address Register Low
.equ	EEDR	=$1D	; EEPROM Data Register
.equ	EECR	=$1C	; EEPROM Control Register
;* PORT B I/O REGISTERS
.equ	PORTB	=$18	; Port B
.equ	DDRB	=$17	; Data Direction Register Port B
.equ	PINB	=$16
;* PORT C I/O REGISTERS 
.equ	PORTC	=$15	; Port C
.equ	DDRC	=$14	; Data Direction Register Port C
.equ	PINC	=$13
;* PORT D I/O REGISTERS
.equ	PORTD	=$12	; Port D
.equ	DDRD	=$11	; Data Direction Register Port D
.equ	PIND	=$10

;* UART I/O REGISTERS
.equ	UDR	=$0C	; UART I/O Data Register
.equ	USR	=$0B	; UART Status Register
.equ	UCR	=$0A	; UART Control Register
.equ	UBRR	=$09	; UART BAUD Rate Register
;* Analog to Digital Converter Registers
.equ	ADMUX	=$07	; ADC Multiplexor Select Register
.equ	ADCSR	=$06	; ADC Control and Status Register
.equ	ADCH	=$05	; ADC Data Register (High)
.equ	ADCL	=$04	; ADC Data Register (Low)
;-= General Definitions/Equates
.def	XL	=r26	; X Index Register Low Byte
.def	XH	=r27	; X Index Register High Byte
.def	YL	=r28	; Y Index Register Low Byte
.def	YH	=r29	; Y Index Register High Byte
.def	ZL	=r30	; Z Index Register Low Byte
.def	ZH	=r31	; Z Index Register High Byte

.equ	OC1A	=5
.equ	OC1B	=4

;-= Application Specific Definitions/Equates

;-= Predefinied SRAM Address Locations
.equ	CPT_H	=$00		; Current Process Table High Address(SRAM)
.equ	CPT_L	=$60		; Current Process Table Low Address(SRAM)
.equ	PS1_H	=$00		; Stack Pointer High for Process 1
.equ	PS1_L	=$A0		; Stack Pointer Low for Process 1
.equ	JPH_T1OVF	=$00B3	; Timer1 Overflow Jump Vector Storage Location (High)
.equ	JPL_T1OVF	=$00B2	; Timer1 Overflow Jump Vector Storage Location (Low)
.equ	JPH_TIM2	=$00B1	; Timer2 Jump Vector Storage Location (High)
.equ	JPL_TIM2	=$00B0	; Timer2 Jump Vector Storage Location (Low)
.equ	MDET_END	=$00C5	; Motion Detector Sweep End
.equ	MDET_ST		=$00C4	; Motion Detector Sweep Start
.equ	NCDS_H		=$00C3	; CDS Calibrated Neutral Value (High)
.equ	NCDS_L		=$00C2	; CDS Calibrated Neutral Value (Low)
.equ	NFSR_H		=$00C1	; Force Sensing Resistor Calibrated Neutral Value (High)
.equ	NFSR_L		=$00C0	; Force Sensing Resistor Calibrated Neutral Value (Low)

;-= Register <-> Symbol Cross-reference Table

;RESERVED	=r0	; RESERVED for UserDebugInterface
.def	SYSTMP3	=r1	; System Temp3
.def	M_TMP	=r2	;
.def	R_Hits	=r3	; Number of Consecutive Right Turns (Reserved for Collision Avoidance)
.def	L_Hits	=r4	; Number of Consecutive Left Turns (Reserved for Collision Avoidance)
.def	T2DLY1	=r5	; Delay Register 1 Strictly Used by Timer2.inc
.def	T2DLY2	=r6	; Delay Register 2 Strictly Used by Timer2.inc
.def	MD_HIT	=r7	; Motion Detector Hit Directional Value
;.def	DET_CW	=r7	; Detected Position going Clockwise
;.def	DET_CCW	=r8	; Detected Position going Counter-Clockwise
.def	MOT_LOC	=r9	; Current Position in the PWM controlling the servo that swings the Motion Detector (Reserved for MotionDetect.inc)
.def	M_DLY1	=r10	; Delay Counter 1 used by Motion Detection
.def	M_DLY2	=r11	; Delay Counter 2 used by Motion Detection
.def	TIM2CNT	=r12	; Register used to save Timer2's value
;UNUSED		=r13-r14	; UNUSED 2 Registers
.def	GENIOR	=r15	; Generic I/O Register for Register to Register
.def	GENIOI	=r16	; Generic I/O Register for Immediate to Register
.def	STATUS	=r17	
	; 	Extended Description:	Used by Various subroutine of the program
	;				it is up to the user to make sure that the usage of status is
	;				completely isolated to that specific subroutine or set of subroutines
	;				in the instruction class			
	;				bit0 - General Timeout bit used in various modules
	;	Collision Avoidance Human Hit Handler
	;				bit2 - Used to indicate Tray Empty Status
	;	Motion Detection:
	;				bit4 - Used to indicate the direction of detection 0=CW,1=CCW
	;				bit3 - Used to indicate Motion Sensor Hit
	; 
;UNUSED		=r20-r21; UNUSED 5 Registers
.def	FUNLED	=r18	; LED Value Storage Register for fun
.def	ND_CNT	=r19	; No Hit Counter
.def	SYSTMP	=r22	; System Temp
.def	SYSTMP2	=r23	; System Temp2
.def	TEMP	=r24	; Temp Register for Internal Calcuations
.def	TEMP2	=r25	; Temp2 Register for Internal Calculations Register to Register

;-= Application Specific Macros
;***********************************************************************************************
;* Macro - CALL_TIMER2 ARG0,ARG1,ARG2
;* Description: Initializes the Z-Index register to the SRAM Address Location of ARG0
;*		and then sets the delay of Timer2 to ARG1,ARG2 where ARG1 is the repeat count
;*		and ARG2 is the 1st time count value
;*		Timer2 will count down ARG2, when ARG2 = 0, ARG1 is decremented
;*		then Timer2 Delay counts down again to 0 but this time it starts at $FF
;*		Example: CALL_TIMER2	CATimeout,$01,$7F
;*			 ARG0 = ijump to CATimeout
;*			 ARG1,ARG2 = $01,$7F ~ 4 seconds 
;***********************************************************************************************
.MACRO		CALL_TIMER2
		ldi ZH, high(@0)
		ldi ZL, low(@0)
		sts JPH_TIM2, ZH
		sts JPL_TIM2, ZL
		ldi GENIOI, @2
		mov GENIOR, GENIOI
		ldi GENIOI, @1
		rcall TIM2_TIMER
.ENDMACRO
;*
;***********************************************************************************************
;* Macro - CALL_TIMER1 ARG0
;* Description: Initializes the Z-Index register to the SRAM Address Location of ARG0
;*		Example: CALL_TIMER2	CATimeout
;***********************************************************************************************
.MACRO		CALL_TIMER1
		ldi ZH, high(@0)
		ldi ZL, low(@0)
		sts JPH_T1OVF, ZH
		sts JPL_T1OVF, ZL
		ldi GENIOI, @1
		rcall TIM1_TIMER
.ENDMACRO

.MACRO		DISP_STRING
		ldi ZH, high(@0)
		ldi ZL, low(@0)
		rcall OutStr2		
.ENDMACRO

.MACRO		GET_ADC
		ldi GENIOI, @0
		rcall ADC_OneRun		
.ENDMACRO
;.MACRO		DISP_STRING
;		ldi ZL, low(@0*2)
;		ldi ZH, high(@0*2)
;		rcall OutStr
;.ENDMACRO