.CSEG
;**************************************************************************************
;* serverBOTAVR: Module -> Collision Avoidance Module
;* Description:	This is the primary module for the serverBOT which handles the
;*		collision avoidance and movement.
;* 3 = FSR
;* 4 = TOPIR
;* 5 = TOPIR
;* 6 = CDS
;**************************************************************************************
Basic_Main:	rcall Init_Motors
		rcall Init_ADC		;
		rcall LEDS_OFF
		cbr STATUS, $04		; Clear Bit2 so full CA_HH is enabled
		
Start_Loop:	ldi GENIOI, $00
		rcall ADC_OneRun
		mov SYSTMP, GENIOI
		cpi SYSTMP, $02
		brlo Start_Loop
		
		ldi ND_CNT, $04
		
Get_Sensor:	rcall Motion_Sweep
		rcall Analyze_MOT
		rcall Test_Motors
		rjmp Get_Sensor
		
;*****************************************************************************************************
CheckTray:	ldi GENIOI, $03
		rcall ADC_8bit
		cpi GENIOI, $08
		brlo NoTray
		lds TEMP, NFSR_L
		cp GENIOI, TEMP
		brlo TrayEmpty
CTray_EX:	ret
		
TrayEmpty:	rcall M_Stop
		sbr STATUS, $04		; Set Bit2 to disable Human Hits on Collision Avoidance
		sbi PORTB, 2		; Turn on Diagnostic LED 1
		sbi PORTC, 0
		cbi PORTC, 0
		rjmp GOHome

NoTray:		rcall M_Stop
		ldi GENIOI, $03
		rcall ADC_8bit
		cpi GENIOI, $08
		brlo NoTray
		rjmp CTray_EX
		
GOHome:		rcall Test_Motors
		ldi GENIOI, $06
		rcall ADC_8bit
		cpi GENIOI, $45
		brlo Terminate
		;lds TEMP, NCDS_L
		;cp GENIOI, TEMP
		;brlo Terminate
		rcall CA_HH
		rjmp GOHome
		
				
Terminate:	rcall M_Stop
infLoop:	rjmp infLoop
;*****************************************************************************************************	
Dispense:	rcall M_Stop
		CALL_TIMER2	CATimeout,$01,$FF
		ldi FUNLED, $84
HH_WT:		sbrc STATUS, 0
		rjmp Dispense_Cont
		sbrc T2DLY2, 4
		rcall LED_PAT1
		rjmp HH_WT
		
Dispense_Cont:	in TEMP, TCNT0
		sbrc TEMP, 1
		rjmp Dispense_TR
		rjmp Dispense_TL
Dispense_EX:	rcall SLEDS_OFF
		ret

Dispense_TR:	in TEMP, TCNT1L
		andi TEMP, $0F
		ldi TEMP2, $19
		add TEMP, TEMP2
		mov M_TMP, TEMP
		rcall M_Turn_Right
		rjmp Dispense_EX
		
Dispense_TL:	in TEMP, TCNT1L
		andi TEMP, $0F
		ldi TEMP2, $19
		add TEMP, TEMP2
		mov M_TMP, TEMP
		rcall M_Turn_Left
		rjmp Dispense_EX
;****************************************************************************************************
;* Subroutine - CA_HH (Collision Avoidance and Human Hit Handler)
;****************************************************************************************************
CA_HH:		sbrc STATUS, 2		; If Status Bit2=1, Tray is Empty
		rjmp NoHit		; Skip HumanHits
		
		rcall CheckTray
		ldi GENIOI, $04
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh HumanHit
		ldi GENIOI, $05
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh HumanHit

NoHit:		ldi GENIOI, $01
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh Right_Turn
		ldi GENIOI, $00
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh Left_Turn
		rjmp CA_Exit

Left_Turn:	sbrc STATUS, 2		; If Status Bit2=1, Tray is Empty
		rjmp CA_IR_L		; Skip HumanHits
		
		ldi GENIOI, $04
		rcall ADC_8bit
		cpi GENIOI, $30
		brsh HumanHit
		ldi GENIOI, $05
		rcall ADC_8bit
		cpi GENIOI, $30
		brsh HumanHit
		
CA_IR_L:	ldi GENIOI, $16
		mov M_TMP, GENIOI
		rcall M_Turn_Left
		ldi GENIOI, $00
		rcall ADC_8bit
		cpi GENIOI, $60
		brlo CA_Exit
		rjmp Left_Turn
		
Right_Turn:	sbrc STATUS, 2		; If Status Bit2=1, Tray is Empty
		rjmp CA_IR_R		; Skip HumanHits
		
		ldi GENIOI, $04
		rcall ADC_8bit
		cpi GENIOI, $30
		brsh HumanHit
		ldi GENIOI, $05
		rcall ADC_8bit
		cpi GENIOI, $30
		brsh HumanHit
		
CA_IR_R:	ldi GENIOI, $16
		mov M_TMP, GENIOI
		rcall M_Turn_Right
		ldi GENIOI, $01
		rcall ADC_8bit
		cpi GENIOI, $60
		brlo CA_Exit
		rjmp Right_Turn

CA_Exit:	ret

HumanHit:	rcall Dispense
		rcall Test_Motors
		rjmp CA_Exit	