.CSEG
;**************************************************************************************	
;* Subroutine - External Interrupt 0 Enable	
;* Description: Enables the External Interrupt 0 and sets it to trigger off of the
;*		rising edge
;**************************************************************************************		
INT0_EN:	push TEMP
		ldi TEMP, $03			; Set INT0 to trigger off of rising edge
		out MCUCR, TEMP
		
		ldi TEMP, $40			; Enable External Interrupt 0
		out GIMSK, TEMP
		pop TEMP
		ret
;**************************************************************************************	
;* Subroutine - External Interrupt 0 Disable
;* Description: Disables the External Interrupt 0	
;**************************************************************************************		
INT0_DIS:	ldi TEMP, $00			; Disable External Interrupt 0
		out GIMSK, TEMP
		ret
;*************************************************************************************
EXT_INT0:	rcall Dispense
		;CALL_TIMER2	CATimeout,$01,$35
		;rcall Test_Motors
		reti
;**************************************************************************************
;* serverBOTAVR: Module -> Collision Avoidance Module
;* Description:	This is the primary module for the serverBOT which handles the
;*		collision avoidance and movement.
;* 3 = FSR
;* 4 = TOPIR
;* 5 = TOPIR
;* 6 = CDS
;**************************************************************************************
CA:		rcall Init_Motors
		rcall Init_ADC		;
		rcall LEDS_OFF
		
Start_Loop:	ldi GENIOI, $00
		rcall ADC_OneRun
		mov SYSTMP, GENIOI
		cpi SYSTMP, $02
		brlo Start_Loop
		
		rjmp Get_Sensor
;*****************************************************************************************
;* Branch Placement
;*****************************************************************************************
NoTray:		rcall M_Stop
		ldi GENIOI, $03
		rcall ADC_8bit
		cpi GENIOI, $08
		brlo NoTray
		rjmp Get_Sensor

HumanHit:	rcall Dispense
		;CALL_TIMER2	CATimeout,$01,$35
		;rcall Test_Motors
		rjmp Get_Sensor	
		
TrayEmpty:	;rcall M_Stop
		sbi PORTB, 2		; Turn on Diagnostic LED 1
		sbi PORTC, 0
		cbi PORTC, 0
infLoop:	rjmp infLoop
;******************************************************************************************				
Get_Sensor:	;GET_ADC $03
		ldi GENIOI, $03
		rcall ADC_8bit
		cpi GENIOI, $08
		brlo NoTray
;		lds TEMP, NFSR_L
;		cp GENIOI, TEMP
;		brlo TrayEmpty
;		rjmp Get_Sensor		;<------ REMOVE	
;		rcall Motion_Sweep
;		rcall Analyze_MOT	

TopIR:		ldi GENIOI, $04
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh HumanHit
		ldi GENIOI, $05
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh HumanHit

NoHit:		ldi GENIOI, $01
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh Right_Turn
		ldi GENIOI, $00
		rcall ADC_8bit
		cpi GENIOI, $60
		brsh Left_Turn
		rcall Test_Motors
		rjmp Get_Sensor

Left_Turn:	;ldi GENIOI, $04
		;rcall ADC_8bit
		;cpi GENIOI, $30
		;brsh HumanHit
		;ldi GENIOI, $05
		;rcall ADC_8bit
		;cpi GENIOI, $30
		;brsh HumanHit
		
		ldi GENIOI, $16
		mov M_TMP, GENIOI
		rcall M_Turn_Left
		ldi GENIOI, $00
		rcall ADC_8bit
		cpi GENIOI, $60
		brlo GS_Again
		rjmp Left_Turn
		
Right_Turn:	;ldi GENIOI, $04
		;rcall ADC_8bit
		;cpi GENIOI, $30
		;brsh HumanHit
		;ldi GENIOI, $05
		;rcall ADC_8bit
		;cpi GENIOI, $30
		;brsh HumanHit
		
		ldi GENIOI, $16
		mov M_TMP, GENIOI
		rcall M_Turn_Right
		ldi GENIOI, $01
		rcall ADC_8bit
		cpi GENIOI, $60
		brlo GS_Again
		rjmp Right_Turn
		
GS_Again:	rcall Test_Motors
;		sbrs STATUS, 0
;		rjmp Get_Sensor
;		rcall M_Stop
;		CALL_TIMER2	CATimeout,$01,$35	;Assembler Macro Instruction
		rjmp Get_Sensor
;*****************************************************************************************************
Dispense:	rcall M_Stop
		CALL_TIMER2	CATimeout,$01,$FF
		ldi FUNLED, $88
HH_WT:		sbrc STATUS, 0
;		ret
		rjmp Dispense_Cont
		sbrc T2DLY2, 4
		rcall LED_PAT1
		rjmp HH_WT
		
Dispense_Cont:	in TEMP, TCNT0
		sbrc TEMP, 1
		rjmp Dispense_TR
		rjmp Dispense_TL
Dispense_EX:	rcall LEDS_OFF
		ret

Dispense_TR:	in TEMP, TCNT1L
		andi TEMP, $0F
		ldi TEMP2, $19
		add TEMP, TEMP2
		mov M_TMP, TEMP
		rcall M_Turn_Right
		rjmp Dispense_EX
		
Dispense_TL:	in TEMP, TCNT1L
		andi TEMP, $0F
		ldi TEMP2, $19
		add TEMP, TEMP2
		mov M_TMP, TEMP
		rcall M_Turn_Left
		rjmp Dispense_EX
;****************************************************************************************************