.CSEG
.equ	MDET_RM		=$21	; Motion Detector Sweep RightMost Value
.equ	MDET_LM		=$0E	; Motion Detector Sweep LeftMost Value
;**************************************************************************************
;* Subroutine - TIM2_PWM
;* PWM - Servo Minimum: $0C
;*	 Servo Maximum: $24
;**************************************************************************************
TIM2_PWM:	sbi DDRD, 7
		ldi TEMP, $66		; PWM, NON-INVERTING, CLK=CLK/256
		out TCCR2, TEMP
		
		;ldi TEMP, $18		; Set OCR2 to ($18) Center Position
		;out OCR2, TEMP		
		
		ret			; Return From Subroutine
;**************************************************************************************		
;* Subroutine - Motion Delay
;* Description: This subroutine is used to create quick pause in between the incremental
;*		sweeps
;* Input: TEMP2
;* Uses: T2DLY1, T2DLY2, TEMP2
;**************************************************************************************		
Motion_Delay:	;ldi TEMP2, $09
MDelay_LP1:	clr M_DLY1		; T2DLY1 = $FF
MDelay_LP2:	clr M_DLY2		; T2DLY2 = $FF
MDelay_LP3:	inc M_DLY2		; T2DLY2 = T2DLY2 - 1
		brne MDelay_LP3		;
		inc M_DLY1		; T2DLY1 = T2DLY1 - 1
		brne MDelay_LP2		;
		dec TEMP2		; TEMP2 = TEMP2 - 1
		brne MDelay_LP1		;
		ret
;**************************************************************************************		
;* Subroutine - Motion_Sweep
;* Description: This subroutine is used to sweep the motion sensor CW then CCW
;*		as the motion detector is being swept, only the first value obtained
;*		in each direction of motion is recorded resulting in two values
;*		one stored in DET_CW, and DET_CCW
;* Uses: TEMP
;* Output: DET_CW, DET_CCW
;**************************************************************************************		
Motion_SWEEP:	;DISP_STRING	nxtline
		cbr STATUS, $08			; Clear Status bit 3
						; When Status bit 3 is set then hit recorded
		cbi PORTB, 3
		sbi PORTC, 0
		cbi PORTC, 0
		
		clr MD_HIT			;
		;mov GENIOI, MD_HIT
		;rcall HEX2ASCII
		;DISP_STRING	nxtline
						
		sbrc STATUS, 4			; If Status bit4=1, Jump to CCW_SETUP
		rjmp CCW_SETUP			; 
CW_SETUP:	ldi GENIOI, MDET_LM		; else CW_SETUP
		sts MDET_ST, GENIOI		; Start at Leftmost				
		ldi GENIOI, MDET_RM		;
		sts MDET_END, GENIOI		; End at RIghtmost
		;		;
SWEEP_ST:	rcall TIM2_PWM			; Set Timer2 to PWM Mode and PortD OC2 to output
		lds MOT_LOC, MDET_ST		; Set Servo to Start Position
		out OCR2, MOT_LOC		; 
		;		
		ldi TEMP2, $08			; Wait until SERVO SWINGS to Start Position
		rcall Motion_Delay		;
		
SWEEP_RST:	rcall CA_HH			; Collision Avoidance Human Hit Handler
		ldi GENIOI, $02			; Wait for RESET value
		rcall ADC_8bit			;
		andi GENIOI, $F0		;
		cpi GENIOI, $80			;
		brne SWEEP_RST			;
		
		sbrc STATUS, 4			; If Status bit4=1,CCW
		dec MOT_LOC			; 	CCW = dec MOT_LOC
		sbrs STATUS, 4			; If Status bit4=0,CW
		inc MOT_LOC			; 	 CW = inc MOT_LOC
SWEEP_LP1:	out OCR2, MOT_LOC		; Incremental Clockwise Servo Swing
		;
		ldi TEMP2, $03			; Delay
		rcall Motion_Delay		;
		rcall CA_HH			; Collision Avoidance Human Hit Handler
		;
		rcall Motion_Detect		; Check Motion_Detector
		sbrc STATUS, 3			; If Status bit3=1, HIT exit SWEEP
		rjmp SWEEP_EX			;		
		;
		sbrc STATUS, 4			; If Status bit4=1, CCW
		dec MOT_LOC			; 	CCW = dec MOT_LOC
		sbrs STATUS, 4			; If Status bit4=0, CW
		inc MOT_LOC			; 	 CW = inc MOT_LOC
		;
		lds TEMP, MDET_END		; TEMP = MDET_END
		cp TEMP, MOT_LOC		; Compare TEMP to MOT_LOC
		brne SWEEP_LP1
		;
		sbrc STATUS, 4			; If Status bit 4=1,CCW
		rjmp CCW_CH			; 	CCW = Jump to CCW_CH
		sbr STATUS, $10			; CW = SET STATUS bit 4
SWEEP_EX:	ret
		;
CCW_CH:		cbr STATUS, $10			; 	CCW = CLR STATUS bit 4
		rjmp SWEEP_EX			; exit
		;
CCW_SETUP:	ldi GENIOI, MDET_RM		; CCW_SETUP
		sts MDET_ST, GENIOI		; Start at Rightmost
		ldi GENIOI, MDET_LM		;	;
		sts MDET_END, GENIOI		; End at LeftMost
		rjmp SWEEP_ST			; Begin Sweep
;*
;**************************************************************************************	
;* Interrupt - External Interrupt 0 Handler
;* Description: When an external interrupt occurs on INT0, a motion is detected by
;*		the sensor. The external Interrupt will check the current direction
;*		and the direction vector to make sure it is the first value for that
;*		direction and if so it records it else it exits
;* Uses: TEMP	
;**************************************************************************************		
Motion_Detect:	push TEMP
		ldi GENIOI, $02
		rcall ADC_8bit
		;push GENIOI
		;rcall HEX2ASCII
		;pop GENIOI
		
		sbrc STATUS, 4			; If status bit4=1, CCW
		rjmp MOTDET_CCW			; 	CCW = Execute CCW Detection

MOTDET_CW:	cpi GENIOI, $92			; $90 = Hit @ MDelay=$03
		brmi MOTDET_EX			; If GENIOI-$A0 < 0, not a valid value

MD_REC:		sbr STATUS, $08			; set status bit3=1 for hit		
		in TEMP, OCR2			; record value into DET_CW
		
		mov MD_HIT, TEMP		;
		sbi PORTB, 3
		sbi PORTC, 0
		cbi PORTC, 0
		
MOTDET_EX:	pop TEMP
		ret
		
MOTDET_CCW:	cpi GENIOI, $70			; 
		brpl MOTDET_EX			; If GENIOI-$6B > 0 not a valid value
		rjmp MD_REC			; record hit value
;*
;**************************************************************************************		
;* Subroutine - ANALYZE_MOT
;* Description:	Used to analyze the values located in DET_CW, DET_CCW, to determine
;*		a general direction to head towards. LEFT, MIDDLE, RIGHT
;**************************************************************************************		
Analyze_MOT:	push TEMP
		push TEMP2
		;DISP_STRING	txtDETECT

		;mov GENIOI, MD_HIT
		;rcall HEX2ASCII
		;DISP_STRING	strResult
		
		tst MD_HIT				; If $00 then no detect
		breq MPOS_ND
		
		ldi ND_CNT, $04
		
		sbrc STATUS, 4
		rjmp CHECK_CCW
		
CHECK_CW:	mov TEMP, MD_HIT
		cpi TEMP, $23
		brsh MDetect_Err
		cpi TEMP, $1F
		brsh MPOS_R
		cpi TEMP, $1C
		brsh MPOS_MR
		cpi TEMP, $18
		brsh MPOS_M
		cpi TEMP, $15
		brsh MPOS_ML
		cpi TEMP, $0E
		brsh MPOS_L
		rjmp MDetect_Err		
		
MAnalyze_EX:	pop TEMP2
		pop TEMP
		ret

MPOS_ND:	;DISP_STRING	strNoDet
		dec ND_CNT
		;mov GENIOI, ND_CNT
		;rcall HEX2ASCII
		tst ND_CNT
		breq MA_RND		
		rjmp MAnalyze_EX
		
MPOS_M:		;DISP_STRING	strM
		rjmp MAnalyze_EX										
								
MDetect_Err:	;DISP_STRING	ERR_Motion
		rjmp MAnalyze_EX
						
MPOS_L:		ldi GENIOI, $0D
		mov M_TMP, GENIOI
		rcall M_Turn_Left
		cbr STATUS, $10			; Set Next Rotation to CW
		;DISP_STRING	strLeft
		rjmp MAnalyze_EX

MPOS_R:		ldi GENIOI, $11
		mov M_TMP, GENIOI
		rcall M_Turn_Right
		sbr STATUS, $10			; Set Next Rotation to CCW
		;DISP_STRING	strRight
		rjmp MAnalyze_EX
		
MPOS_ML:	ldi GENIOI, $08
		mov M_TMP, GENIOI
		rcall M_Turn_Left
		cbr STATUS, $10			; Set Next Rotation to CW
		;DISP_STRING	strML
		rjmp MAnalyze_EX
	
MPOS_MR:	ldi GENIOI, $0A
		mov M_TMP, GENIOI
		rcall M_Turn_Right
		sbr STATUS, $10			; Set Next Rotation to CCW
		;DISP_STRING	strMR
		rjmp MAnalyze_EX
		
MA_RND:		rcall M_TurnRnd
		ldi ND_CNT, $04
		rjmp MAnalyze_EX
		
CHECK_CCW:	mov TEMP, MD_HIT
		cpi TEMP, $22
		brsh MDetect_Err
		cpi TEMP, $1B
		brsh MPOS_R
		cpi TEMP, $17
		brsh MPOS_MR
		cpi TEMP, $14
		brsh MPOS_M
		cpi TEMP, $11
		brsh MPOS_ML
		cpi TEMP, $0E
		brsh MPOS_L
		rjmp MDetect_Err
;**************************************************************************************				
.ESEG
txtDETECT:	
		.db	"DETECTED: "
		.db	EOS
txtCCW:
		.db	"CCW "
		.db	EOS
txtCW:
		.db	"CW "
		.db	EOS
strResult:
		.db	" Result: "
		.db	EOS
strRight:
		.db	"R"
		.db	CR,LF,EOS
strLeft:
		.db	"L"
		.db	CR,LF,EOS
strML:
		.db	"ML"
		.db	CR,LF,EOS
strMR:		
		.db	"MR"
		.db	CR,LF,EOS
strM:
		.db	"M"
		.db	CR,LF,EOS
strNoDet:
		.db	"No Detect"
		.db	CR,LF,EOS
ERR_Motion:
		.db	"Error aquiring motion sensor"
		.db	CR,LF,EOS