.CSEG
;**************************************************************************************
;* Subroutine - TIM2_PWM
;* PWM - Servo Minimum: $0C
;*	 Servo Maximum: $24
;**************************************************************************************
TIM2_PWM:	sbi DDRD, 7
		ldi TEMP, $66		; PWM, NON-INVERTING, CLK=CLK/256
		out TCCR2, TEMP
		
		;ldi TEMP, $18		; Set OCR2 to ($18) Center Position
		;out OCR2, TEMP		
		
		ret			; Return From Subroutine
;**************************************************************************************		
;* Subroutine - Motion Delay
;* Description: This subroutine is used to create quick pause in between the incremental
;*		sweeps
;* Input: TEMP2
;* Uses: T2DLY1, T2DLY2, TEMP2
;**************************************************************************************		
Motion_Delay:	;ldi TEMP2, $09
MDelay_LP1:	clr M_DLY1		; T2DLY1 = $FF
MDelay_LP2:	clr M_DLY2		; T2DLY2 = $FF
MDelay_LP3:	inc M_DLY2		; T2DLY2 = T2DLY2 - 1
		brne MDelay_LP3		;
		inc M_DLY1		; T2DLY1 = T2DLY1 - 1
		brne MDelay_LP2		;
		dec TEMP2		; TEMP2 = TEMP2 - 1
		brne MDelay_LP1		;
		ret
;**************************************************************************************		
;* Subroutine - Motion_Sweep
;* Description: This subroutine is used to sweep the motion sensor CW then CCW
;*		as the motion detector is being swept, only the first value obtained
;*		in each direction of motion is recorded resulting in two values
;*		one stored in DET_CW, and DET_CCW
;* Uses: TEMP
;* Output: DET_CW, DET_CCW
;**************************************************************************************		
Motion_SWEEP:	rcall Init_ADC
		sbr STATUS, $04			; Setup STATUS to show CW Sweep
		clr DET_CW			;
		clr DET_CCW			;
		rcall TIM2_PWM			; Set Timer2 to PWM Mode and PortD OC2 to output

		
		
		ldi TEMP, $0E			; Set Servo to Left Most Position
		mov MOT_LOC, TEMP
		out OCR2, MOT_LOC		; 
				
		ldi TEMP2, $08			;
		rcall Motion_Delay		; Wait until SERVO SWINGS to leftmost position
		
		inc MOT_LOC			; 	
SWEEP_LP1:	out OCR2, MOT_LOC		; Incremental Clockwise Servo Swing
		
		ldi TEMP2, $08
		rcall Motion_Delay
		;
		rcall Motion_Detect		; Check Motion_Detector
		;
		inc MOT_LOC			;
		mov TEMP, MOT_LOC
		cpi TEMP, $21			; Check for Rightmost Position
		brne SWEEP_LP1
			
		ldi TEMP2, $0C
		rcall Motion_Delay
		dec MOT_LOC
		cbr STATUS, $01			; Setup STATUS to show CCW Sweep
SWEEP_LP2:	out OCR2, MOT_LOC		;
		
		ldi TEMP2, $08
		rcall Motion_Delay
;
		rcall Motion_Detect		; Check Motion_Detector
;
		dec MOT_LOC
		mov TEMP, MOT_LOC		;
		cpi TEMP, $0E			; Check for Leftmost position
		brne SWEEP_LP2
		
		ret
;*
;**************************************************************************************	
;* Interrupt - External Interrupt 0 Handler
;* Description: When an external interrupt occurs on INT0, a motion is detected by
;*		the sensor. The external Interrupt will check the current direction
;*		and the direction vector to make sure it is the first value for that
;*		direction and if so it records it else it exits
;* Uses: TEMP	
;**************************************************************************************		
Motion_Detect:	push TEMP
		sbrc STATUS, 0			; If status bit0 = 0 then execute CCW
		rjmp MOTDET_CW			; else execute CW
		
MOTDET_CCW:	tst DET_CCW			; Check to see if First Value of CCW
		brne MOTDET_EX			; If not exit else record first value on CCW

		ldi GENIOI, $02			;
		rcall ADC_OneRun		; Call ADC_OneRun with ADMUX set to port2
		cpi GENIOI, $00			; $00			;
		;brne MOTDET_EX			; If GENIOI != 0 then bad value and exit
		brne CCW_ALT			; If GENIOI != 0 then bad value and exit

CCW_REC:	in TEMP, OCR2			; record first value on CCW
		mov DET_CCW, TEMP

MOTDET_EX:	pop TEMP			;
		reti

CCW_ALT:	cpi GENIOI, $01			; Secondary Threshold for CCW = $01 72
		brne MOTDET_EX			; if higher byte != $01 then exit
		mov GENIOI, GENIOR		; 
		cpi GENIOI, $AF			; Check if lower byte < $72
		brlo CCW_REC			;
		rjmp MOTDET_EX			; If not exit

MOTDET_CW:	tst DET_CW			; Check to see if First value of CW
		brne MOTDET_EX			; If not exit else record first value on CW

		ldi GENIOI, $02			; Call ADC_OneRun with ADMUX set to port2
		rcall ADC_OneRun		; 
		cpi GENIOI, $03			; $03		
		;brne MOTDET_EX			; If GENIOI != 0 then bad value and exit
		brne CW_ALT			; If GENIOI != 0 then bad value and exit

CW_REC:		in TEMP, OCR2			; record first value on CW
		mov DET_CW, TEMP

		rjmp MOTDET_EX
CW_ALT:		cpi GENIOI, $02			; Secondary Threshold for CCW = $02 80
		brne MOTDET_EX			; if higher byte != $01 then exit
		mov GENIOI, GENIOR		; 
		cpi GENIOI, $80			; Check if lower byte < $80
		brlo MOTDET_EX			; If so exit
		rjmp CW_REC			; else record value
;*
;**************************************************************************************		
;* Subroutine - ANALYZE_MOT
;* Description:	Used to analyze the values located in DET_CW, DET_CCW, to determine
;*		a general direction to head towards. LEFT, MIDDLE, RIGHT
;**************************************************************************************		
Analyze_MOT:	push TEMP
		push TEMP2
		
CHECK_CC:	mov TEMP, DET_CW
		clr DET_CW
		
CH_UPPER_CW:	cpi TEMP, $21
		brpl MDetect_Err
		inc DET_CW
		
CH_HIGH_CW:	cpi TEMP, $1A
		brlo CH_MIDDLE_CW
		lsl DET_CW
		
CH_MIDDLE_CW:	cpi TEMP, $16
		brlo CH_LOWER_CW
		lsl DET_CW
		
CH_LOWER_CW:	cpi TEMP, $0E
		;brlo MDetect_Err
		brlo CHECK_CCW
		lsl DET_CW
						
		;tst DET_CCW
		;breq MDetect_Err
		
CHECK_CCW:	mov TEMP, DET_CCW
		clr DET_CCW
		
CH_UPPER_CCW:	cpi TEMP, $21
		brpl MDetect_Err
		inc DET_CCW
		
CH_HIGH_CCW:	cpi TEMP, $1A
		brlo CH_MIDDLE_CCW
		lsl DET_CCW
		
CH_MIDDLE_CCW:	cpi TEMP, $16
		brlo CH_LOWER_CCW
		lsl DET_CCW
		
CH_LOWER_CCW:	cpi TEMP, $0E
		;brlo MDetect_Err
		brlo MA_NEXT
		lsl DET_CCW
		
		rcall Init_Motors
MA_NEXT:	mov TEMP, DET_CW
		or TEMP, DET_CCW	; TEMP = DET_CW OR DET_CCW
		
		cpi TEMP, $02
		breq MPOS_L
		cpi TEMP, $06
		breq MPOS_ML
		cpi TEMP, $0C
		breq MPOS_MR
		cpi TEMP, $08
		breq MPOS_R
		andi TEMP, $01
		cpi TEMP, $01
		breq MPOS_ND
						
MAnalyze_EX:	pop TEMP2
		pop TEMP
		ret
		
MDetect_Err:	rjmp MAnalyze_EX
						
MPOS_L:		ldi GENIOI, $1A
		mov M_TMP, GENIOI
		rcall M_Turn_Left
		rjmp MAnalyze_EX
		
MPOS_ML:	ldi GENIOI, $10
		mov M_TMP, GENIOI
		rcall M_Turn_Left
		rjmp MAnalyze_EX
	
MPOS_MR:	ldi GENIOI, $10
		mov M_TMP, GENIOI
		rcall M_Turn_Right
		rjmp MAnalyze_EX
		
MPOS_R:		ldi GENIOI, $1A
		mov M_TMP, GENIOI
		rcall M_Turn_Right
		rjmp MAnalyze_EX
		
MPOS_ND:	;ldi GENIOI, $25
		;mov M_TMP, GENIOI
		;rcall M_Turn_Right
		rjmp MAnalyze_EX
;**************************************************************************************	