.CSEG
;**************************************************************************************
;* serverBOTAVR: Module -> Motor Controller Module
;* Description: This module controls the movement of the motors.
;**************************************************************************************
;***************************************************************************
;* SUBROUTINE - Init_Motors
;* Sets OC1A and OC1B on portD as outputs
;* The TimerCLK is set to CLK/256 = 8Mhz/256 = 31.250Khz
;* The PWM has 510 possible values so that the total period is 16.32ms
;* 1-bit = .032 ms
;*
;* Uses: TEMP
;***************************************************************************
Init_Motors:	sbi DDRD, OC1A		; portD pin 5(OC1A) = output
		sbi DDRD, OC1B		; portD pin 4(OC1B) = output
		
		ldi TEMP, $A1		; Setups non-inverting 8-bit pwm on
		out TCCR1A, TEMP	; channels A & B
		
		ldi TEMP, $04		; Timer1CLK = CLK/256
		out TCCR1B, TEMP	;
		
		ret			; Return from Subroutine
;*
;***************************************************************************
;* Subroutine - M_Turn_Right
;* 
;* Description: Makes the Motors Turn Right for the amount of time located
;*		in M_TMP
;* Input: M_TMP
;* Uses: TEMP, TEMP2
;***************************************************************************
M_Turn_Right:	ldi TEMP2, $00
		out OCR1AH, TEMP2
		out OCR1BH, TEMP2
		ldi TEMP2, $17
		out OCR1AL, TEMP2
		out OCR1BL, TEMP2		
MtR_LP1:	ldi TEMP, $FF
MtR_LP2:	ldi TEMP2, $FF
MtR_LP3:	dec TEMP2
		brne MtR_LP3
		dec TEMP
		brne MtR_LP2
		dec M_TMP
		brne MtR_LP1
		ldi TEMP2, $00
		out OCR1AL, TEMP2
		out OCR1BL, TEMP2
		ret
;*
;***************************************************************************
;* Subroutine - M_Turn_Left
;* 
;* Description: Makes the Motors Turn Left for the amount of time located
;*		in M_TMP
;* Input: M_TMP
;* Uses: TEMP, TEMP2
;***************************************************************************
M_Turn_Left:	ldi TEMP2, $00
		out OCR1AH, TEMP2
		out OCR1BH, TEMP2
		ldi TEMP2, $19
		out OCR1AL, TEMP2
		out OCR1BL, TEMP2		
MtL_LP1:	ldi TEMP, $FF
MtL_LP2:	ldi TEMP2, $FF
MtL_LP3:	dec TEMP2
		brne MtL_LP3
		dec TEMP
		brne MtL_LP2
		dec M_TMP
		brne MtL_LP1
		ldi TEMP2, $00
		out OCR1AL, TEMP2
		out OCR1BL, TEMP2
		ret
;*
;***************************************************************************
;* Subroutine - M_Stop
;* Description: Stops the Motors
;* Uses: TEMP2
;***************************************************************************
M_Stop:		ldi TEMP2, $00
		out OCR1AH, TEMP2
		out OCR1AL, TEMP2
		out OCR1BH, TEMP2
		out OCR1BL, TEMP2
		
		ret
;*
;***************************************************************************
;* 58 - Counter Clockwise 15, 17, 
;* 5C - Center = 18
;* 66 - Clockwise = 19 ->
;***************************************************************************
Test_Motors:	ldi TEMP2, $00		;
		out OCR1AH, TEMP2
		out OCR1BH, TEMP2
		
		ldi TEMP2, $20		; Counter Clockwise on A
		out OCR1AL, TEMP2	;
		
		ldi TEMP2, $16		; Clockwise on B
		out OCR1BL, TEMP2	;
		
		ret
;****************************************************************************
;* Subroutine - M_Turn Random
;*		
M_TurnRnd:	in TEMP, TCNT0
		sbrc TEMP, 1
		rjmp MTurnRnd_R
		rjmp MTurnRnd_L
M_TurnRnd_EX:	ret

MTurnRnd_R:	in TEMP, TCNT1L
		andi TEMP, $0F
		ldi TEMP2, $23
		add TEMP, TEMP2
		mov M_TMP, TEMP
		rcall M_Turn_Right
		rjmp M_TurnRnd_EX
		
MTurnRnd_L:	in TEMP, TCNT1L
		andi TEMP, $0F
		ldi TEMP2, $20
		add TEMP, TEMP2
		mov M_TMP, TEMP
		rcall M_Turn_Left
		rjmp M_TurnRnd_EX