;*************************************************************************
;*
;* ServerBOT V1.0
;* Derrick Moy
;* EEL5666C - IMDL SPRING 2002
;* 
;*************************************************************************
.LIST
.LISTMAC
;* DEVICE DEFINITION FILE
.include "avr8535.inc"
;*
;*************************************************************************
;* BEGIN CODE
;*************************************************************************
.CSEG
;* Reset and Interrupt Jump Vectors
.ORG $0000
		rjmp RESET
.ORG $0004
		rjmp TIM2_OVF
.ORG $0008
		rjmp TIM1_OVF
.ORG $0009
		rjmp TIM0_OVF
;.ORG $000B
;		rjmp UART_RX
;*
;*************************************************************************
;* Main
;*************************************************************************	
.ORG $000e
RESET: 		ldi r16, $00		; Setup Stack Pointer to enable
		out SPH, r16		; subroutine calls
		ldi r16, $90		;
		out SPL, r16		;

; Initialize the Current Process Table (CPT) and set the current process
; to 0

		ldi YH, CPT_H		; Load YH to CPT_H
		ldi YL, CPT_L		; Load YL to CPT_L
		clr TEMP		; Set TEMP to 0
		st Y, TEMP		;

; Setup Timer0 to interrupt on overflow and set the clk for timer0 to
; CLK/1024
						
		ldi TEMP, $01		;
		out TIMSK, TEMP		;

		ldi TEMP, $01		; $05, CLK/1024
		out TCCR0, TEMP		;

; Enable Global Interrupts and jump to process 0
		rcall Init_LEDS
		rcall LEDS_OFF
		ldi FUNLED, $55
		rcall Init_ADC		
		sei			; Enable Global Interrupts
	
		CALL_TIMER2	CalibratePS,$01,$7A
		cbi PORTB, 3
		sbi PORTB, 2		; Turn on Diagnostic LED 1
		sbi PORTC, 0
		cbi PORTC, 0
CPS_WT:		sbrc STATUS, 0
		rjmp CCDS
		sbrc T2DLY2, 4
		rcall SLEDS_OFF
		sbrs T2DLY2, 4
		rcall SLEDS_ON
		rjmp CPS_WT		

CCDS:		sbi PORTB, 3		; Turn on Diagnostic LED 2
		cbi PORTB, 2		; Turn off Diagnostic LED 1
		sbi PORTC, 0
		cbi PORTC, 0
		CALL_TIMER2	CalibrateCDS,$01,$7A
CCDS_WT:	sbrc STATUS, 0
		rjmp Cal_Done
		sbrc T2DLY2, 4
		rcall SLEDS_OFF
		sbrs T2DLY2, 4
		rcall SLEDS_ON
		rjmp CCDS_WT		

		rcall Init_UART
Cal_Done:	rjmp Basic_Main
		;rjmp DisplayMnu
; Test and Debug Stuff (Deletable)
;		rcall Init_Motors
;FollowMe:	rcall Motion_SWEEP
;		rcall Analyze_MOT
;		CALL_TIMER2	CATimeOut,$01,$5B
;FOLMEWT:	sbrs STATUS, 0
;		rjmp FOLMEWT		
;		rjmp FollowMe
				
SLEDS_ON:	in TEMP, PORTB
		ori TEMP, $F0
		out PORTB, TEMP
		sbi PORTC, 0
		cbi PORTC, 0
		ret

SLEDS_OFF:	in TEMP, PORTB
		andi TEMP, $0F
		out PORTB, TEMP
		sbi PORTC, 0
		cbi PORTC, 0
		ret
				
LEDS_OFF:	in TEMP, PORTB
		andi TEMP, $00
		out PORTB, TEMP
		sbi PORTC, 0
		cbi PORTC, 0
		ret		
		
LED_PAT1:	ror FUNLED
		mov TEMP, FUNLED
		andi TEMP, $F0
		out PORTB, TEMP
		sbi PORTC, 0
		cbi PORTC, 0
		ret			
;*
;*************************************************************************
;* Interrupts
;*************************************************************************	
;* Interrupt - Timer0 Overflow
;* CPT - [NumPS|CurPS]
;*        7   4 3   0
;*************************************************************************	
TIM0_OVF:	in SYSTMP, SREG		; Store Status Register and push
		push SYSTMP		; Status Register Onto Stack
		
		push TEMP		;Store TEMP
		push TEMP2		;Store TEMP2
		push GENIOI		;Store GENIOI
		push GENIOR		;Store GENIOR
		
		ldi YH, CPT_H		; Load Y as Current Process Table
		ldi YL, CPT_L		;
		ld SYSTMP2, Y+		; 
		
		andi SYSTMP2, $0F	; Isolate Current PS
		lsl SYSTMP2		; Offset Current PS by Mult 2
		
		add YL, SYSTMP2		; Add Offset to Y-Index
		
		in SYSTMP3, SPH
		st Y+, SYSTMP3		; Store Current Stack Pointer into Table
		in SYSTMP3, SPL
		st Y, SYSTMP3		;
		
		ldi YH, CPT_H		;
		ldi YL, CPT_L		;
		ld SYSTMP, Y		; Reload PS
		mov SYSTMP2, SYSTMP	; 
		
		swap SYSTMP2		; Swap to get Num PS
		andi SYSTMP2, $0F	; Isolate Num PS
		breq TIM0_NEXT		; If NumPS=0 Exit

		andi SYSTMP, $0F	; Isolate Current PS
		cp SYSTMP, SYSTMP2	; Compare Current and NumPS
		brne CPS_INC		; If Current and NumPS not equal
		clr SYSTMP		; reset Current to 0, else increment current
		rjmp TIM0_NEXT		;

CPS_INC:	inc SYSTMP		;
TIM0_NEXT:	swap SYSTMP2		; 
		or SYSTMP, SYSTMP2	; Reform Process Entry
		st Y+, SYSTMP		; Store Process into Y and increment Y

TIM0_OVF_EX:	andi SYSTMP, $0F	; Isolate Current PS
		lsl SYSTMP		; Offset SYSTMP: Mult 2
		add YL, SYSTMP		; Add Offset to Y-Index
		ld SYSTMP3, Y+		; Load new Stack PointerLow
		out SPH, SYSTMP3
		ld SYSTMP3, Y		; Load new Stack PointerHigh					
		out SPL, SYSTMP3
		
		pop GENIOR		; Restore GENIOR		
		pop GENIOI		; Restore GENIOI
		pop TEMP2		; Restore TEMP2
		pop TEMP		; Restore TEMP

		pop SYSTMP		; Pop Status Register
		out SREG, SYSTMP	; Restore Status Register
		reti 

;*
;*************************************************************************	
;* SUBROUTINE MODULE LINKER - USAGE: .include "<MODULE>"
;*
;* UserDebugInterface.inc	-> User Debug Interface
;*
;* motor_control.inc 		-> Motor Driver via PWM on OC1A and OC1B
;*
;* colavoid.inc			-> Collision Avoidance
;* 				   Dependencies: motor_control.inc
;*						 ADC.inc
;* motiondetect.inc		-> Motion Detector Controller
;*				   Dependencies: timer2.inc
;*
;*************************************************************************
.include "UserDebugInterface.inc"
.include "colavoid.inc"
;.include "motiondetect.inc"
.include "mdetect_diag.inc"
.include "timer2.inc"
.include "motor_control.inc"
.include "ADC.inc"
;*
;*************************************************************************
;* Local Subroutines
;*************************************************************************
;* Subroutine - Start Process
;* Input: SYSTMP(New Process PC High), SYSTMP2(New Process PC Low)
;* Output: Setups the new CPT and adds an entry to the Process/PC Table
;* Uses: SYSTMP, SYSTMP2, X-Index, Y-Index
;*************************************************************************
Begin_PS:	ldi YH, PS1_H		; Set YH to Process1 Stack High
		ldi YL, PS1_L		; Set YL to Process1 Stack Low
		st Y, SYSTMP2		;
		st -Y, SYSTMP		; Load PC
		in SYSTMP, SREG		;
		st -Y, SYSTMP		; Load SREG
		ldi SYSTMP, $FF		;
		st -Y, SYSTMP		; Load TEMP
		st -Y, SYSTMP		; Load TEMP2
		st -Y, SYSTMP		; Load GENIOI
		st -Y, SYSTMP		; Load GENIOR
		dec YL			; decrement stack 1 more
		
		ldi XH, CPT_H		; Load Current Process Table
		ldi XL, CPT_L		;
		
		ld SYSTMP, X+		; Load CPT into SYSTMP
		swap SYSTMP		; Swap SYSTMP to extract Num PS
		andi SYSTMP, $0F	; Isolate NumPS
		inc SYSTMP		; Increment SYSTMP
		mov SYSTMP2, SYSTMP	; Copy new NumPS to SYSTMP2
		lsl SYSTMP		; Offset SYSTMP by Mult 2
		add XL, SYSTMP		; Add Offset to XL
		st X+, YH		; Store New Stack Pointer into X
		st X, YL		;
		
		ldi XH, CPT_H		; 
		ldi XL, CPT_L		;
		swap SYSTMP2		; Swap SYSTMP to correct Polarity
		ld SYSTMP, X		;
		andi SYSTMP, $0F	; Isolate CurrentPS and Clear NumPS
		or SYSTMP, SYSTMP2	; Stitch New NumPS and into CPT
		st X, SYSTMP		;
		
		ret			; Return From Subroutine
;************************************************************************
KillPS:		cli			; Disable Interrupts
		ldi XH, CPT_H		; Re-initialize CPT to 00
		ldi XL, CPT_L
		ldi SYSTMP, $00
		st X, SYSTMP		; Store CPT
		sei			; Re-enable Interrupts
		ret