.CSEG
;**************************************************************************************
;* serverBOTAVR: Module -> Timer2 Controller
;* Description: The Timer2 Controller Module Initializes Timer2 and allows it to switch
;*		back and forth between PWM mode for controlling a 3rd servo and timer
;*		mode for timer wait states.
;**************************************************************************************
TIM1_TIMER:	mov T2DLY1, GENIOI
		clr TEMP		; Disable PWM Mode
		out TCCR1A, TEMP	;
		out TCCR1B, TEMP	; Stop Timer1
		
		clr TEMP		; Clear Timer1 Count Value
		out TCNT1H, TEMP	;
		out TCNT1L, TEMP	;
		
		in TEMP, TIMSK		; Load TEMP with TIMSK
		ori TEMP, $04		; Enable TIM1_OVF Interrupt
		out TIMSK, TEMP		;
		
		ldi TEMP, $04		; Timer1CLK = CLK/256
		out TCCR1B, TEMP	;
		
		ret
;**************************************************************************************
;* Subroutine - TIM1_Disable
;**************************************************************************************
TIM1_Disable:	in TEMP, TIMSK		; Disable TIM1_OVF Interrupt
		andi TEMP, $FB		;
		out TIMSK, TEMP		;
		
		clr TEMP		; Stop Timer1
		out TCCR1B, TEMP	;
		ret
;**************************************************************************************
;* Interrupt - TIM1_OVF
;**************************************************************************************
TIM1_OVF:	dec T2DLY1		;
		brne T1OVF_EX		;
		lds ZH, JPH_T1OVF	; Load ZH with TIM2 pseudo jump vector
		lds ZL, JPL_T1OVF	; Load ZL with TIM2 pseudo jump vector
		ijmp			; Jump to TIM2 PSEUDO JUMP VECTOR
T1OVF_EX:	reti
;*
;**************************************************************************************
;* Subroutine - TIM2_TIMER
;* Initializes Timer2 as a Timer. GENIOI sets the timer2 delay
;* Input: GENIOI, GENIOR, [JPH_TIM2+JPL_TIM2] (Timer2 Timeout JumpVector)
;* Description: Initializes Timer2 to use the delays specified in GENIOI, and GENIOR
;*		Upon Timer2 Timeout, The TIM2_OVF Interrupt Handler jumps and executes
;*		the code pointed to by the JPH_TIM2,JPL_TIM2 Pseudo Jump Vectors
;**************************************************************************************
TIM2_TIMER:	cbr STATUS, $01		; clear the timeout bit in the status register
		cbi DDRD, 7		; Set PortD to input
		rcall TIM2_CLR		; Clear Timer2 and Reset all flags
		mov T2DLY2, GENIOR	;
		mov T2DLY1, GENIOI
		
		in TEMP, TIMSK		; Load TEMP with TIMSK
		ori TEMP, $40		; Enable TIM2_OVF Interrupt
		out TIMSK, TEMP		;
		
		ldi TEMP, $07		; Disabled PWM, Disabled OC, CLK=CLK/1024
		out TCCR2, TEMP		;
		
		ret
;*
;**************************************************************************************
;* Subroutine - TIM2_CLR
;**************************************************************************************
TIM2_CLR:	clr TEMP		;
		out TCCR2, TEMP		; Stop Timer2 and shutoff all functionality
		out TCNT2, TEMP		; Clear TCNT2
		mov T2DLY1, TEMP	; Clear Timer2 Delay Counter 1
		mov T2DLY2, TEMP	; Clear Timer2 Delay Counter 2
		
		in TEMP, TIFR		; Clear TIM2_OVF flag
		andi TEMP, $40		;
		out TIFR, TEMP		;
		ret
;*
;**************************************************************************************
;* Subroutine - TIM2_Disable
;**************************************************************************************
TIM2_Disable:	in TEMP, TIMSK		; Disable TIM2_OVF Interrupt
		andi TEMP, $BF		; 
		out TIMSK, TEMP		; 
		
		clr TEMP		; Stop Timer2
		out TCCR2, TEMP		;
		ret
;*
;**************************************************************************************
;* Interrupt - TIM2_OVF
;**************************************************************************************
TIM2_OVF:	dec T2DLY2		; decrement delay counter
		brne T2OVF_EX		; check of delay counter is 0
		dec T2DLY1		; decrement multiplicity counter
		brne T2OVF_EX		; check of multiplicity counter = 0
		
		lds ZH, JPH_TIM2	; Load ZH with TIM2 pseudo jump vector
		lds ZL, JPL_TIM2	; Load ZL with TIM2 pseudo jump vector
		ijmp			; Jump to TIM2 PSEUDO JUMP VECTOR
T2OVF_EX:	reti
;*
;**************************************************************************************
CATimeout:	sbr STATUS, $01
		in TEMP, TIMSK		; Disable TIM2_OVF Interrupt
		andi TEMP, $BF
		out TIMSK, TEMP
		rjmp T2OVF_EX		
;**************************************************************************************
MOT_Timeout:	sbr STATUS, $02
		rcall TIM1_Disable
		rjmp T1OVF_EX
;**************************************************************************************
CalibratePS:	sbr STATUS, $01
		ldi GENIOI, $03		; set ADMUX to 03 for FSR
		rcall ADC_8bit
		sts NFSR_L, GENIOI
	;rcall ADC_OneRun	; call ADC
	;sts NFSR_H, GENIOI	; Save results in memory
	;sts NFSR_L, GENIOR
		rjmp T2OVF_EX		; exit
;**************************************************************************************
CalibrateCDS:	sbr STATUS, $01
		ldi GENIOI, $06		; set ADMXU to 06 for CDS
		rcall ADC_8bit
		sts NCDS_L, GENIOI	
	;rcall ADC_OneRun	; call ADC
	;sts NCDS_H, GENIOI	; Save Results in memory
	;sts NCDS_L, GENIOR	;
		rjmp T2OVF_EX		; exit
			